#include "Header.hh"
#include "Block.hh"
#include <iostream>
#include <fstream>
#include <string>
#include <vector>

using std::string;
using namespace Header;
 
namespace Block
{

//   DataBlock::DataBlock(string survey)
  DataBlock::DataBlock(SURVEY surveyType)
  {
    headerString = string(" ",headerLength);
    Header::SurveySchema *temp = new Header::SurveySchema(surveyType);
//     Header::SurveySchema *temp = new Header::SurveySchema(survey);
    this->survey = *temp;
    this->data = new char[dataLength];
//     this->surveyName = survey;
    delete temp;
  }
  //--------------------------------------------------------------------------

  DataBlock::~DataBlock()
  {
    delete [] data;
  }
  //--------------------------------------------------------------------------

  bool DataBlock::readBlock(std::ifstream &file)
  {
    if(!this->readHeader(file)) return false;
    this->readData(file);
    return true;
  }
  //--------------------------------------------------------------------------

  bool DataBlock::readHeader(std::ifstream &file)
  {
    char *hdr = new char[headerLength];
    file.read( hdr, headerLength*sizeof(char) );
    if(file.eof()) return false;
    for(int c=0;c<headerLength;c++) this->headerString[c] = hdr[c];
    return true;
  }
  //--------------------------------------------------------------------------

  void DataBlock::readData(std::ifstream &file)
  {
    file.read( data, dataLength*sizeof(char) );
  }
  //--------------------------------------------------------------------------

//   template <typename Type> 
//   Type DataBlock::getKeyword(int field, int element)
//   {
//     Type output = 
//       this->survey.readParameter<Type>(this->headerString,field,element);
//     return output;
//   }
//   template int DataBlock::getKeyword<int>(int field, int element);
//   template float DataBlock::getKeyword<float>(int field, int element);
//   template string DataBlock::getKeyword<string>(int field, int element);
  //--------------------------------------------------------------------------

  template <class T> 
  T DataBlock::getKeyword(int field, int element)
  {
    int dim = this->survey.getFieldDimension(field);
    std::stringstream ss;
    std::vector<T> output(dim);
    for(int i=0;i<dim;i++){
      int start = this->survey.getStartPosition(field)+i*dim;
      int length = this->survey.getFieldLength(field);
      ss.str(this->headerString.substr(pos,length));
      ss >> output[i];      
    }
    return output[elementNumber];
  }

  template<> int DataBlock::getKeyword<int>(int field, int element){
    return this->survey.readParameter<int>(this->headerString,field,element);
  }
  template<> float DataBlock::getKeyword<float>(int field, int element){
    return this->survey.readParameter<float>(this->headerString,field,element);
  }
  template<> double DataBlock::getKeyword<double>(int field, int element){
    return this->survey.readParameter<double>(this->headerString,field,element);
  }
  template<> string DataBlock::getKeyword<string>(int field, int element){
   return this->survey.readParameter<string>(this->headerString,field,element);
  }
  //--------------------------------------------------------------------------
  template <class T> 
  std::vector<T> DataBlock::getFullKeyword(int field)
  {
    int dim = this->survey.getFieldDimension(field);
    std::stringstream ss;
    std::vector<T> output(dim);
    for(int i=0;i<dim;i++){
      int start = this->survey.getStartPosition(field)+i*dim;
      int length = this->survey.getFieldLength(field);
      ss.str(this->headerString.substr(pos,length));
      ss >> output[i];      
    }
    return output;
  }
  template<> std::vector<int> DataBlock::getFullKeyword<int>(int field){
    std::vector<int> output(this->survey.getFieldDimension(field));
    for(int i=0;i<this->survey.getFieldDimension(field);i++)
      output[i]=this->survey.readParameter<int>(this->headerString,field,i);
    return output;
  }
  template<> std::vector<float> DataBlock::getFullKeyword<float>(int field){
    std::vector<float> output(this->survey.getFieldDimension(field));
    for(int i=0;i<this->survey.getFieldDimension(field);i++)
      output[i]= this->survey.readParameter<float>(this->headerString,field,i);
    return output;
  }
  template<> std::vector<string> DataBlock::getFullKeyword<string>(int field){
    std::vector<string> output(this->survey.getFieldDimension(field));
    for(int i=0;i<this->survey.getFieldDimension(field);i++)
      output[i]=this->survey.readParameter<string>(this->headerString,field,i);
    return output;
  }

}
