#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include "../Block.hh"
#include "../Header.hh"
#include "../PSRFits.hh"

using namespace Header;
using namespace Block;
using namespace FITS;
using std::string;

std::string getFileName(int number)
{
  std::stringstream ss;
  ss << "!testfull" << std::setw(2) << std::setfill('0') << number << ".fits";
  return ss.str();
}

int main(int argc, char * argv[])
{
  std::string fname;
  if(argc==1) fname = "../data/s70_tape_test_S181_S18101.dat";
  else if(argc==2) fname = argv[1];
  else{
    std::cerr << "Either \"testfulls70.x\" or \"testfulls70.x filename\"...\n";
    return 1;
  }
  //  std::string fname = "../data/pttape_test";
  SURVEY currentSurvey = idSurvey(fname);

  if(currentSurvey == PM){
    std::cerr <<"\aOops! This is a PM survey!\n Good, but not for this program\nExiting...\n";
  }
  if(currentSurvey != S70){
    if(currentSurvey!=PM)
      std::cerr <<"\aOops! This isn't an S70 survey!\nExiting...\n";
    return 1;
  }
  else std::cerr <<"Yes! This is an S70 survey file. Go ahead...\n";

  std::ifstream fin;
  fin.open(fname.c_str(), std::ios::binary);

  bool eof;
  DataBlock block(currentSurvey);
  block.readBlock(fin);
  
  std::string currentRA = block.getKeyword<std::string>(RA_STRT);

  int filecount = 0;
  PSRFits fitsfile(getFileName(filecount++));
  fitsfile.writePHU(block);
  
  int blockcount = 0;
  do{

    if(block.getKeyword<int>(MOVE)!=MOVING) {
      std::cerr << "block " << blockcount++;
      fitsfile.writeSubint(block);
      std::cerr << " done\n";
    }
    eof = !block.readBlock(fin);

    std::cerr << currentRA << " "
	      << block.getKeyword<std::string>(RA_STRT) << " "
	      << block.getKeyword<int>(MOVE) <<" ";

    if(!eof){

      if((block.getKeyword<std::string>(RA_STRT) != currentRA)
	 || (block.getKeyword<int>(MOVE)==MOVING)){
	std::cerr << "Changing FITS file -- " << filecount << "\n";
	fitsfile.closeFile();
      }

      if((fitsfile.getFilestate()==CLOSED)
	 &&(block.getKeyword<int>(MOVE)==POINT)){
	fitsfile.openFile(getFileName(filecount++));
	fitsfile.writePHU(block);
	currentRA = block.getKeyword<std::string>(RA_STRT);
      }

    }
	
	//work out position
	//    if(!goAgain || position changed) goAgain = false;
	
  }while(!eof);
  
  fitsfile.closeFile();
  fin.close();

}
